@@ -60,6 +60,8 @@ void check_algos();
 
 typedef struct sigaction tSigAct;
 
+cleaner g_victor;
+
 #ifdef HAVE_DAEMON
 inline bool fork_away()
 {
@@ -225,7 +227,6 @@ static void usage(int retCode) {
 		"-c: configuration directory\n"
 		"-e: on startup, run expiration once\n"
 		"-p: print configuration and exit\n"
-		"--retest string: regular expression tester"
 #if SUPPWHASH
 		"-H: read a password from STDIN and print its hash\n"
 #endif
