@@ -2,6 +2,7 @@
 //#define LOCAL_DEBUG
 #include "debug.h"
 
+#include "cacheman.h"
 #include "expiration.h"
 #include "lockable.h"
 #include "acfg.h"
@@ -465,15 +466,23 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 		{
 			// another special case, slightly ugly :-(
 			// this is explicit hardcoded repair code
-			// for the removal of .bz2 compressed versions
-			// of various files
-			if (endsWithSzAr(sFilePathRel, "/Packages.bz2")
-					|| endsWithSzAr(sFilePathRel, "/Sources.bz2")
-					|| (StrHas(sFilePathRel, "i18n/Translation-") &&
-							endsWithSzAr(sFilePathRel, ".bz2")))
-
+			static struct {
+				string fromEnd, toEnd, extraCheck;
+			} fixmap[] =
 			{
-				SendChunkSZ("Attempting to download the .xz version... ");
+					{ "/Packages.bz2", "/Packages.xz", "" },
+					{ "/Sources.bz2", "/Sources.xz", "" },
+					{ "/Release", "/InRelease", "" },
+					{ "/InRelease", "/Release", "" },
+					{ ".bz2", ".xz", "i18n/Translation-" },
+					{ "/Packages.gz", "/Packages.xz", "" },
+					{ "/Sources.gz", "/Sources.xz", "" }
+			};
+			for(const auto& fix : fixmap)
+			{
+				if(!endsWith(sFilePathRel, fix.fromEnd) || !StrHas(sFilePathRel, fix.extraCheck))
+					continue;
+				SendChunkSZ("Attempting to download the alternative version... ");
 				// if we have it already, use it as-is
 				if (!pResolvedDirectUrl)
 				{
@@ -481,23 +490,23 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 					if (p && parserHead.SetHttpUrl(p))
 						pResolvedDirectUrl = &parserHead;
 				}
-				if (pResolvedDirectUrl)
+				auto newurl(*pResolvedDirectUrl);
+				if(!endsWith(newurl.sPath, fix.fromEnd) || !StrHas(newurl.sPath, fix.extraCheck))
+					continue;
+				newurl.sPath.replace(newurl.sPath.size()-fix.fromEnd.size(), fix.fromEnd.size(),
+						fix.toEnd);
+				if(Download(sFilePathRel.substr(0, sFilePathRel.size() - fix.fromEnd.size())
+						+ fix.toEnd, bIsVolatileFile, msgVerbosityLevel, tFileItemPtr(), &newurl,
+				hints&~DL_HINT_GUESS_REPLACEMENT ))
 				{
-					// XXX: maybe pointless copy, work around with a flag?
-					auto newurl(*pResolvedDirectUrl);
-					if (endsWithSzAr(newurl.sPath, ".bz2"))
-					{
-						newurl.sPath.erase(newurl.sPath.size() - 4);
-						newurl.sPath.append(".xz");
-						if(Download(sFilePathRel.substr(0, sFilePathRel.size() - 4) + ".xz",
-								bIsVolatileFile, msgVerbosityLevel, tFileItemPtr(), &newurl))
-						{
-							MarkObsolete(sFilePathRel);
-							return true;
-						}
-						return false;
-					}
+					MarkObsolete(sFilePathRel);
+					return true;
 				}
+				// XXX: this sucks a little bit since we don't want to show the checkbox
+				// when the fallback download succeeded... but on failures, the previous one
+				// already added a newline before
+				AddDelCbox(sFilePathRel, true);
+				return false;
 			}
 		}
 
@@ -1183,8 +1192,9 @@ void tCacheOperation::UpdateVolatileFiles()
 	{
 		cmstring sPathRel=relRef.first+relRef.second;
 
-		if(!Download(sPathRel, true, m_metaFilesRel[sPathRel].hideDlErrors
-					? eMsgHideErrors : eMsgShow))
+		if(!Download(sPathRel, true,
+				m_metaFilesRel[sPathRel].hideDlErrors ? eMsgHideErrors : eMsgShow,
+						tFileItemPtr(), 0, DL_HINT_GUESS_REPLACEMENT))
 		{
 			m_nErrorCount+=(!m_metaFilesRel[sPathRel].hideDlErrors);
 
@@ -2117,15 +2127,25 @@ void tCacheOperation::ProcessSeenMetaFiles(ifileprocessor &pkgHandler)
 	}
 }
 
-void tCacheOperation::AddDelCbox(cmstring &sFileRel)
+void tCacheOperation::AddDelCbox(cmstring &sFileRel, bool bExtra)
 {
 	auto ref_isnew = m_delCboxFilter.insert(sFileRel);
 	if(! ref_isnew.second)
 		return;
 
-	SendFmtRemote <<  "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
-			<< "\" value=\"" << sFileRel << "\">Tag</label>";
-
+	if(bExtra)
+	{
+		string bn(GetBaseName(sFileRel));
+		if(startsWithSz(bn, "/"))
+			bn.erase(0, 1);
+		SendFmtRemote <<  "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
+					<< "\" value=\"" << sFileRel << "\">(also tag " << bn << ")</label><br>";
+	}
+	else
+		SendFmtRemote <<  "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
+			<< "\" value=\"" << sFileRel << "\">Tag</label>"
+			"\n<!--\n" maark << int(ControLineType::Error) << "Problem with "
+			<< sFileRel << "\n-->\n";
 }
 void tCacheOperation::TellCount(uint nCount, off_t nSize)
 {
@@ -2149,31 +2169,61 @@ void tCacheOperation::PrintStats(cmstring &title)
 {
 	multimap<off_t, cmstring*> sorted;
 	off_t total=0;
-	const uint nMax = m_bVerbose ? (UINT_MAX-1) : MAX_TOP_COUNT;
-	uint hidden=0;
 	for(auto &f: m_metaFilesRel)
 	{
 		total += f.second.space;
 		if(f.second.space)
 			EMPLACE_PAIR(sorted,f.second.space, &f.first);
-		if(sorted.size()>nMax)
-		{
-			sorted.erase(sorted.begin());
-			hidden++;
-		}
 	}
 	if(!total)
 		return;
-	m_fmtHelper << "<br>\n<table><tr><td colspan=2><u>" << title;
-	if(!m_bVerbose && hidden>0)
-		m_fmtHelper << " (Top " << sorted.size() << ", " << hidden <<  " more not displayed)";
-	m_fmtHelper << "</u></td></tr>\n";
+	int nMax = std::min(int(sorted.size()), int(MAX_TOP_COUNT));
+	if(m_bVerbose)
+		nMax = MAX_VAL(int);
+
+	if(!m_bVerbose)
+	{
+	m_fmtHelper << "<br>\n<table name=\"shorttable\"><thead>"
+			"<tr><th colspan=2>" << title;
+	if(!m_bVerbose && sorted.size()>MAX_TOP_COUNT)
+		m_fmtHelper << " (Top " << nMax << "<span name=\"noshowmore\">,"
+				" <a href=\"javascript:show_rest();\">show more / cleanup</a></span>)";
+	m_fmtHelper << "</th></tr></thead>\n<tbody>";
 	for(auto it=sorted.rbegin(); it!=sorted.rend(); ++it)
 	{
-		m_fmtHelper << "<tr><td><b>" << offttosH(it->first) << "</b></td><td>"
+		m_fmtHelper << "<tr><td><b>"
+				<< offttosH(it->first) << "</b></td><td>"
 				<< *(it->second) << "</td></tr>\n";
+		if(nMax--<=0)
+			break;
 	}
-	SendFmt << "</table>";
+	SendFmt << "</tbody></table>"
+
+	// the other is hidden for now
+	<< "<div name=\"bigtable\" class=\"xhidden\">";
+	}
+
+	m_fmtHelper << "<br>\n<table><thead>"
+				"<tr><th colspan=1><input type=\"checkbox\" onclick=\"copycheck(this, 'xfile');\"></th>"
+				"<th colspan=2>" << title << "</th></tr></thead>\n<tbody>";
+		for(auto it=sorted.rbegin(); it!=sorted.rend(); ++it)
+		{
+			m_fmtHelper << "<tr><td><input type=\"checkbox\" class=\"xfile\" name=\"kf" << nKillLfd++
+					<< "\" value=\"" << *(it->second) << "\"></td>"
+						"<td><b>" << offttosH(it->first) << "</b></td><td>"
+					<< *(it->second) << "</td></tr>\n";
+
+
+		}
+		SendFmt << "</tbody></table>";
+
+		if(m_delCboxFilter.empty())
+			SendFmtRemote << "<br><b>Action(s):</b><br>"
+							"<input type=\"submit\" name=\"doDelete\""
+							" value=\"Delete selected files\">";
+
+		if(!m_bVerbose)
+			SendFmt << "</div>";
 }
 
 #ifdef DEBUG
