@@ -29,8 +29,9 @@ static cmstring sGenericError("567 Unknown download error occured");
 
 std::atomic_uint g_nDlCons(0);
 
-dlcon::dlcon(bool bManualExecution, string *xff) :
-		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nTempPipelineDisable(0),
+dlcon::dlcon(bool bManualExecution, string *xff, IDlConFactory *pConFactory) :
+		m_pConFactory(pConFactory), m_bStopASAP(false), m_bManualMode(bManualExecution),
+		m_nTempPipelineDisable(0),
 		m_bProxyTot(false)
 {
 	LOGSTART("dlcon::dlcon");
@@ -736,7 +737,7 @@ dlcon::~dlcon()
   g_nDlCons--;
 }
 
-inline uint dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueue &inpipe)
+inline uint dlcon::ExchangeData(mstring &sErrorMsg, tDlStreamHandle &con, tDljQueue &inpipe)
 {
 	LOGSTART2("dlcon::ExchangeData",
 			"qsize: " << inpipe.size() << ", sendbuf size: "
@@ -900,10 +901,11 @@ inline uint dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
                 nTakesPerSec=1;
              m_nSpeedLimitMaxPerTake = nSpeedNowKib*1024/nTakesPerSec;
              auto nIntervalUS=1000000 / nTakesPerSec;
+             auto nIntervalUS_copy = nIntervalUS;
              // creating a bitmask
              for(m_nSpeedLimiterRoundUp=1,nIntervalUS/=2;nIntervalUS;nIntervalUS>>=1)
                 m_nSpeedLimiterRoundUp = (m_nSpeedLimiterRoundUp<<1)|1;
-
+             m_nSpeedLimitMaxPerTake = uint(double(m_nSpeedLimitMaxPerTake) * double(m_nSpeedLimiterRoundUp) / double(nIntervalUS_copy));
           }
           // waiting for the next time slice to get data from buffer
           timeval tv;
@@ -1078,7 +1080,7 @@ void dlcon::WorkLoop()
 	}
 
 	tDljQueue inpipe;
-	tTcpHandlePtr con;
+	tDlStreamHandle con;
 	uint loopRes=0;
 
 	bool bStopRequesting=false; // hint to stop adding request headers until the connection is restarted
@@ -1103,10 +1105,7 @@ void dlcon::WorkLoop()
 		{
 			return cjob->m_pRepoDesc->m_pProxy;
 		}
-		if(!acfg::proxy_info.sHost.empty())
-			return &acfg::proxy_info;
-
-		return nullptr;
+		return acfg::GetProxyInfo();
 	};
 
 	while(true) // outer loop: jobs, connection handling
@@ -1125,7 +1124,7 @@ void dlcon::WorkLoop()
         		if(inpipe.empty())
         		{
         			if(con)
-        				tcpconnect::RecycleIdleConnection(con);
+        				m_pConFactory->RecycleIdleConnection(con);
         			return;
         		}
         	}
@@ -1165,7 +1164,7 @@ void dlcon::WorkLoop()
 
 				auto doconnect = [&](const tHttpUrl& tgt, int timeout, bool fresh)
 				{
-					return tcpconnect::CreateConnected(tgt.sHost,
+					return m_pConFactory->CreateConnected(tgt.sHost,
 							tgt.GetPort(),
 							sErrorMsg,
 							&bUsed,
@@ -1175,7 +1174,7 @@ void dlcon::WorkLoop()
 			}	;
 
 				auto& cjob = m_qNewjobs.front();
-				auto proxy = m_bProxyTot ? nullptr : prefProxy(cjob);
+				auto proxy = prefProxy(cjob);
 				auto& peerHost = cjob->GetPeerHost();
 
 #ifdef HAVE_SSL
@@ -1211,6 +1210,7 @@ void dlcon::WorkLoop()
         			ldbg("optional proxy broken, disable");
         			m_bProxyTot = true;
         			proxy = nullptr;
+				acfg::MarkProxyFailure();
         			con = doconnect(peerHost, acfg::nettimeout, false);
         		}
 
@@ -1323,7 +1323,7 @@ void dlcon::WorkLoop()
 			if (con && !(loopRes & all_err))
 			{
 				dbgline;
-				tcpconnect::RecycleIdleConnection(con);
+				m_pConFactory->RecycleIdleConnection(con);
 				continue;
 			}
 		}
@@ -1347,7 +1347,7 @@ void dlcon::WorkLoop()
         	// reinsert them into the new task list and continue
 
         	// if conn was not reset above then it should be in good shape
-        	tcpconnect::RecycleIdleConnection(con);
+        	m_pConFactory->RecycleIdleConnection(con);
         	goto move_jobs_back_to_q;
         }
 
