@@ -97,10 +97,12 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry)
 			Cstat realState(SABSPATH(sPathRel));
 			if(!realState)
 			{
-				SendFmt << WCLASS "File not accessible, ignoring " << sPathRel << CLASSEND;
-				return false;
+				SendFmt << WCLASS "File not accessible, will remove metadata of " << sPathRel << CLASSEND;
+				m_forceKeepInTrash[sPathRel]=true;
+				return true;
 			}
-			off_t lenFromStat = realState.st_size;
+			auto lenFromStat = realState.st_size;
+
 			//SendFmt << "DBG-disk-size: " << lenFromStat;
 
 			// those file were not updated by index handling, and are most likely not
@@ -410,7 +412,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 
 			if (ContHas(m_forceKeepInTrash, sPathRel))
 			{
-				LOG("forcetrash flag set, whitelist does not apply, not to be removed");
+				LOG("forcetrash flag set, whitelist does not apply, shall be removed");
 			}
 			else if (Match(fileGroup.first, FILE_WHITELIST) || Match(sPathRel, FILE_WHITELIST))
 			{
@@ -586,7 +588,7 @@ void expiration::Action()
 
 	PrintStats("Allocated disk space");
 
-	SendChunk("<br>Done.</br>");
+	SendChunk("<br>Done.");
 
 	save_fail_count:
 
@@ -807,10 +809,9 @@ inline bool expiration::CheckAndReportError()
 {
 	if (m_nErrorCount > 0 && m_bErrAbort)
 	{
-		SendFmt << sAbortMsg <<
-				((m_nPrevFailCount+(m_nErrorCount>0) > acfg::exsupcount)
-				? "<!-- TELL:THE:ADMIN -->"
-						: "<!-- NOT:TELLING:THE:ADMIN:YET -->");
+		SendFmt << sAbortMsg;
+		if(m_nPrevFailCount+(m_nErrorCount>0) > acfg::exsupcount)
+			SendFmt << "\n<!--\n" maark << int(ControLineType::Error) << "Errors found, aborting expiration...\n-->\n";
 		return true;
 	}
 	return false;
