@@ -83,7 +83,7 @@ void tSpecialRequest::SendChunkRemoteOnly(const char *data, size_t len)
 void tSpecialRequest::SendChunk(const char *data, size_t len)
 {
 	SendChunkRemoteOnly(data, len);
-	AfterSendChunk(data, len);
+	SendChunkLocalOnly(data, len);
 }
 
 void tSpecialRequest::SendChunkedPageHeader(const char *httpstatus, const char *mimetype)
@@ -114,7 +114,7 @@ public:
         "Not Authorized. Please contact Apt-Cacher NG administrator for further questions.<br>"
         "<br>"
         "For Admin: Check the AdminAuth option in one of the *.conf files in Apt-Cacher NG "
-        "configuration directory, probably /etc/apt-cacher-ng/." ;
+        "configuration directory, probably " CFGDIR  ;
 		SendRawData(authmsg, sizeof(authmsg)-1, 0);
 	}
 };
@@ -238,9 +238,15 @@ tSpecialRequest::eMaintWorkType tSpecialRequest::DispatchMaintWork(cmstring& cmd
 	// all of the following need authorization if configured, enforce it
 	switch(acfg::CheckAdminAuth(auth))
 	{
-	case 0: break; // auth is ok or no passwort is set
-	case 1: return workAUTHREQUEST;
-	default: return workAUTHREJECT;
+     case 0:
+#ifdef HAVE_CHECKSUM
+        break; // auth is ok or no passwort is set
+#else
+        // most data modifying tasks cannot be run safely without checksumming support 
+        return workAUTHREJECT;
+#endif
+     case 1: return workAUTHREQUEST;
+     default: return workAUTHREJECT;
 	}
 
 	struct { LPCSTR trigger; tSpecialRequest::eMaintWorkType type; } matches [] =
