@@ -37,12 +37,13 @@ atomic_int nConCount(0), nDisconCount(0), nReuseCount(0);
 #include <openssl/err.h>
 #endif
 
-std::atomic_uint tcpconnect::g_nconns(0);
+std::atomic_uint dl_con_factory::g_nconns(0);
+dl_con_factory g_tcp_con_factory;
 
 tcpconnect::tcpconnect(acfg::tRepoData::IHookHandler *pObserver) : m_pStateObserver(pObserver)
 {
 	if(acfg::maxdlspeed != RESERVED_DEFVAL)
-		g_nconns.fetch_add(1);
+		dl_con_factory::g_nconns.fetch_add(1);
 	if(pObserver)
 		pObserver->OnAccess();
 }
@@ -52,7 +53,7 @@ tcpconnect::~tcpconnect()
 	LOGSTART("tcpconnect::~tcpconnect, terminating outgoing connection class");
 	Disconnect();
 	if(acfg::maxdlspeed != RESERVED_DEFVAL)
-		g_nconns.fetch_add(-1);
+		dl_con_factory::g_nconns.fetch_add(-1);
 #ifdef HAVE_SSL
 	if(m_ctx)
 	{
@@ -103,7 +104,7 @@ void termsocket(int fd)
 	};
 }
 
-static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t addrlen, time_t timeout)
+static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t addrlen, time_t timeout, bool bAssumeNonBlock)
 {
 	long stflags;
 	struct timeval tv;
@@ -113,13 +114,15 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 	tv.tv_sec = timeout;
 	tv.tv_usec = 0;
 
-	if ((stflags = fcntl(sockfd, F_GETFL, nullptr)) < 0)
-		return -1;
-
-	// Set to non-blocking mode.
-	if (fcntl(sockfd, F_SETFL, stflags|O_NONBLOCK) < 0)
-		return -1;
+	if(!bAssumeNonBlock)
+	{
+		if ((stflags = fcntl(sockfd, F_GETFL, nullptr)) < 0)
+			return -1;
 
+		// Set to non-blocking mode.
+		if (fcntl(sockfd, F_SETFL, stflags | O_NONBLOCK) < 0)
+			return -1;
+	}
 	res = connect(sockfd, addr, addrlen);
 	if (res < 0) {
 		if (EINPROGRESS == errno)
@@ -161,8 +164,7 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 		}
 	}
 
-	// Set back to original mode, which may or may not have been blocking.
-	if (fcntl(sockfd, F_SETFL, stflags) < 0)
+	if(!bAssumeNonBlock && fcntl(sockfd, F_SETFL, stflags) < 0) // Set back to original mode
 		return -1;
 
 	return 0;
@@ -208,18 +210,21 @@ inline bool tcpconnect::_Connect(string & sErrorMsg, int timeout)
 				::setsockopt(m_conFd, IPPROTO_TCP, TCP_NODELAY, &yes, sizeof(yes));
 			}
 #endif
-
-			if (::connect_timeout(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen, timeout) < 0)
+			set_nb(m_conFd);
+			if (::connect_timeout(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen, timeout, true) < 0)
 			{
 				if(errno==ETIMEDOUT)
 					sErrorMsg="Connection timeout";
+#ifndef MINIBUILD
+				USRDBG(tErrnoFmter("Outgoing connection for ") << m_sHostName << ", Port: " << m_sPort );
+#endif
 				continue;
 			}
 #ifdef DEBUG
 			nConCount.fetch_add(1);
 #endif
 			ldbg("connect() ok");
-			set_nb(m_conFd);
+
 			return true;
 		}
 	}
@@ -254,16 +259,16 @@ void tcpconnect::Disconnect()
 }
 lockable spareConPoolMx;
 multimap<tuple<string,string SSL_OPT_ARG(bool) >,
-		std::pair<tTcpHandlePtr, time_t> > spareConPool;
+		std::pair<tDlStreamHandle, time_t> > spareConPool;
 
-tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
+tDlStreamHandle dl_con_factory::CreateConnected(cmstring &sHostname, cmstring &sPort,
 		mstring &sErrOut, bool *pbSecondHand, acfg::tRepoData::IHookHandler *pStateTracker
 		,bool bSsl, int timeout, bool nocache)
 {
 	LOGSTART2s("tcpconnect::CreateConnected", "hostname: " << sHostname << ", port: " << sPort
 			<< (bSsl?" with ssl":" , no ssl"));
 
-	tTcpHandlePtr p;
+	tDlStreamHandle p;
 #ifndef HAVE_SSL
 	if(bSsl)
 	{
@@ -337,7 +342,7 @@ tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
 	return p;
 }
 
-void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
+void dl_con_factory::RecycleIdleConnection(tDlStreamHandle & handle)
 {
 	if(!handle)
 		return;
@@ -390,7 +395,7 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	handle.reset();
 }
 
-time_t tcpconnect::BackgroundCleanup()
+time_t dl_con_factory::BackgroundCleanup()
 {
 	lockguard __g(spareConPoolMx);
 	time_t now=GetTime();
@@ -439,7 +444,7 @@ void tcpconnect::KillLastFile()
 #endif
 }
 
-void tcpconnect::dump_status()
+void dl_con_factory::dump_status()
 {
 	lockguard __g(spareConPoolMx);
 	tSS msg;
@@ -483,13 +488,14 @@ bool tcpconnect::SSLinit(mstring &sErr, cmstring &sHostname, cmstring &sPort)
 		SSL_CTX_load_verify_locations(m_ctx,
 				acfg::cafile.empty() ? nullptr : acfg::cafile.c_str(),
 			acfg::capath.empty() ? nullptr : acfg::capath.c_str());
-
 	}
 
 	ssl = SSL_new(m_ctx);
 	if(!ssl)
 		goto ssl_init_fail;
 
+	SSL_set_tlsext_host_name(ssl, sHostname.c_str());
+
 	// mark it connected and prepare for non-blocking mode
  	SSL_set_connect_state(ssl);
  	SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY
@@ -659,3 +665,4 @@ bool tcpconnect::StartTunnel(const tHttpUrl& realTarget, mstring& sError,
 	return true;
 }
 
+
